from typing import AsyncGenerator

from symphony.bdk.core.auth.auth_session import AuthSession
from symphony.bdk.core.config.model.bdk_retry_config import BdkRetryConfig
from symphony.bdk.core.retry import retry
from symphony.bdk.core.service.pagination import offset_based_pagination
from symphony.bdk.gen.agent_api.share_api import ShareApi
from symphony.bdk.gen.agent_model.share_content import ShareContent
from symphony.bdk.gen.agent_model.v2_message import V2Message
from symphony.bdk.gen.pod_api.room_membership_api import RoomMembershipApi
from symphony.bdk.gen.pod_api.streams_api import StreamsApi
from symphony.bdk.gen.pod_model.membership_list import MembershipList
from symphony.bdk.gen.pod_model.room_detail import RoomDetail
from symphony.bdk.gen.pod_model.stream import Stream
from symphony.bdk.gen.pod_model.stream_attributes import StreamAttributes
from symphony.bdk.gen.pod_model.stream_filter import StreamFilter
from symphony.bdk.gen.pod_model.stream_list import StreamList
from symphony.bdk.gen.pod_model.user_id import UserId
from symphony.bdk.gen.pod_model.user_id_list import UserIdList
from symphony.bdk.gen.pod_model.v2_admin_stream_filter import V2AdminStreamFilter
from symphony.bdk.gen.pod_model.v2_admin_stream_info import V2AdminStreamInfo
from symphony.bdk.gen.pod_model.v2_admin_stream_list import V2AdminStreamList
from symphony.bdk.gen.pod_model.v2_member_info import V2MemberInfo
from symphony.bdk.gen.pod_model.v2_membership_list import V2MembershipList
from symphony.bdk.gen.pod_model.v2_room_search_criteria import V2RoomSearchCriteria
from symphony.bdk.gen.pod_model.v2_stream_attributes import V2StreamAttributes
from symphony.bdk.gen.pod_model.v3_room_attributes import V3RoomAttributes
from symphony.bdk.gen.pod_model.v3_room_detail import V3RoomDetail
from symphony.bdk.gen.pod_model.v3_room_search_results import V3RoomSearchResults


class OboStreamService:
    """Class exposing OBO-enabled endpoints for stream management."""

    def __init__(self, streams_api: StreamsApi, room_membership_api: RoomMembershipApi, share_api: ShareApi,
                 auth_session: AuthSession, retry_config: BdkRetryConfig):
        """

        :param streams_api: a generated StreamsApi instance.
        :param room_membership_api: a generated RoomMembershipApi instance.
        :param share_api: a generated ShareApi instance.
        :param auth_session: the bot session.
        """
        self._streams_api = streams_api
        self._room_membership_api = room_membership_api
        self._share_api = share_api
        self._auth_session = auth_session
        self._retry_config = retry_config

    @retry
    async def get_stream(self, stream_id: str) -> V2StreamAttributes:
        """Returns information about a particular stream.
        Wraps the `Stream Info V2 <https://developers.symphony.com/restapi/reference#stream-info-v2>`_ endpoint.

        :param stream_id: the ID of the stream to be retrieved.
        :return: the information about the given stream.
        """
        return await self._streams_api.v2_streams_sid_info_get(sid=stream_id,
                                                               session_token=await self._auth_session.session_token)

    @retry
    async def list_streams(self, stream_filter: StreamFilter, skip: int = 0,
                           limit: int = 50) -> StreamList:
        """Returns a list of all the streams of which the requesting user is a member,
        sorted by creation date (ascending - oldest to newest).
        Wraps the `List User Streams <https://developers.symphony.com/restapi/reference#list-user-streams>`_ endpoint.

        :param stream_filter: the stream searching criteria.
        :param skip: number of stream results to skip.
        :param limit: maximum number of streams to return.
        :return: the list of stream retrieved matching the search filter.
        """
        return await self._streams_api.v1_streams_list_post(filter=stream_filter, skip=skip, limit=limit,
                                                            session_token=await self._auth_session.session_token)

    @retry
    async def list_all_streams(self, stream_filter: StreamFilter, chunk_size: int = 50, max_number: int = None) \
            -> AsyncGenerator[StreamAttributes, None]:
        """Returns an asynchronous of all the streams of which the requesting user is a member,
        sorted by creation date (ascending - oldest to newest).
        Wraps the `List User Streams <https://developers.symphony.com/restapi/reference#list-user-streams>`_ endpoint.

        :param stream_filter:  the stream searching criteria.
        :param chunk_size: the maximum number of elements to retrieve in one underlying HTTP call
        :param max_number: the total maximum number of elements to retrieve
        :return: an asynchronous generator of the streams matching the search filter.
        """

        async def list_streams_one_page(skip, limit):
            result = await self.list_streams(stream_filter, skip, limit)
            return result.value if result else None

        return offset_based_pagination(list_streams_one_page, chunk_size, max_number)

    @retry
    async def add_member_to_room(self, user_id: int, room_id: str):
        """Adds a member to an existing room.
        Wraps the `Add Member <https://developers.symphony.com/restapi/reference#add-member>`_ endpoint.

        :param user_id: the id of the user to be added to the room.
        :param room_id: the id of the room in which to add the user.
        :return: None
        """
        await self._room_membership_api.v1_room_id_membership_add_post(
            payload=UserId(id=user_id), id=room_id,
            session_token=await self._auth_session.session_token)

    @retry
    async def remove_member_from_room(self, user_id: int, room_id: str):
        """Removes a member from an existing room.
        Wraps the `Remove Member <https://developers.symphony.com/restapi/reference#remove-member>`_ endpoint.

        :param user_id: the id of the user to be removed from the room.
        :param room_id: the id of the room from which to remove the user.
        :return: None
        """
        await self._room_membership_api.v1_room_id_membership_remove_post(
            payload=UserId(id=user_id), id=room_id,
            session_token=await self._auth_session.session_token)

    @retry
    async def share(self, stream_id: str, content: ShareContent) -> V2Message:
        """Share third-party content, such as a news article, into the specified stream.
        The stream can be a chat room, an IM, or an MIM.
        Wraps the `Share <https://developers.symphony.com/restapi/reference#share-v3>`_ endpoint.

        :param stream_id: the id of the stream in which to share the given content.
        :param content: the third-party content to be shared.
        :return: the created message.
        """
        return await self._share_api.v3_stream_sid_share_post(
            sid=stream_id, share_content=content, session_token=await self._auth_session.session_token,
            key_manager_token=await self._auth_session.key_manager_token)

    @retry
    async def promote_user_to_room_owner(self, user_id: int, room_id: str):
        """Promotes user to owner of the chat room.
        Wraps the `Promote Owner <https://developers.symphony.com/restapi/reference#promote-owner>`_ endpoint.

        :param user_id: the id of the user to be promoted as owner.
        :param room_id: the room id.
        :return: None
        """
        await self._room_membership_api.v1_room_id_membership_promote_owner_post(
            id=room_id, payload=UserId(id=user_id), session_token=await self._auth_session.session_token)

    @retry
    async def demote_owner_to_room_participant(self, user_id: int, room_id: str):
        """Demotes room owner to a participant in the chat room.
        Wraps the `Demote Owner <https://developers.symphony.com/restapi/reference#demote-owner>`_ endpoint.

        :param user_id: the id of the room owner to be demoted.
        :param room_id: the room id.
        :return: None
        """
        await self._room_membership_api.v1_room_id_membership_demote_owner_post(
            id=room_id, payload=UserId(id=user_id), session_token=await self._auth_session.session_token)


class StreamService(OboStreamService):
    """Service class to manage streams.
    """

    @retry
    async def create_im_or_mim(self, user_ids: [int]) -> Stream:
        """Create a new single or multi party instant message conversation between the caller and specified users.
        The caller is implicitly included in the members of the created chat.
        Duplicate users will be included in the membership of the chat but the duplication will be silently ignored.
        If there is an existing IM conversation with the same set of participants then the id of that existing stream
        will be returned.
        If the given list of user ids contains only one id, an IM will be created, otherwise, a MIM will be created.

        Wraps the `Create IM or MIM <https://developers.symphony.com/restapi/reference#create-im-or-mim>`_ endpoint.

        :param user_ids: the list of user ids ti be put as room participants.
        :return: the created stream.
        """
        return await self._streams_api.v1_im_create_post(uid_list=UserIdList(value=user_ids),
                                                         session_token=await self._auth_session.session_token)

    @retry
    async def create_room(self, room_attributes: V3RoomAttributes) -> V3RoomDetail:
        """Creates a new chatroom.
        If no  attributes are specified, the room is created as a private chatroom.
        Wraps the `Create Room V3 <https://developers.symphony.com/restapi/reference#create-room-v3>`_ endpoint.

        :param room_attributes: attributes of the room to be created.
        :return: details of created room.
        """
        return await self._streams_api.v3_room_create_post(payload=room_attributes,
                                                           session_token=await self._auth_session.session_token)

    @retry
    async def search_rooms(self, query: V2RoomSearchCriteria, skip: int = 0,
                           limit: int = 50) -> V3RoomSearchResults:
        """Search for rooms according to the specified criteria.
        Wraps the `Search Rooms V3 <https://developers.symphony.com/restapi/reference#search-rooms-v3>`_ endpoint.

        :param query: the search criteria.
        :param skip: number of rooms to skip, defaults to 0.
        :param limit: number of maximum rooms to return. Must be a positive integer that does not exceed 100.
        :return: the rooms matching search criteria.
        """
        return await self._streams_api.v3_room_search_post(query=query, skip=skip, limit=limit,
                                                           session_token=await self._auth_session.session_token)

    async def search_all_rooms(self, query: V2RoomSearchCriteria, chunk_size: int = 50,
                               max_number: int = None) -> AsyncGenerator[V3RoomDetail, None]:
        """Search for rooms according to the specified criteria.
        Wraps the `Search Rooms V3 <https://developers.symphony.com/restapi/reference#search-rooms-v3>`_ endpoint.

        :param query: the search criteria.
        :param chunk_size: the maximum number of elements to retrieve in one underlying HTTP call.
        :param max_number: the total maximum number of elements to retrieve.
        :return: an asynchronous generator of the rooms matching the search criteria.
        """

        async def search_rooms_one_page(skip, limit):
            result = await self.search_rooms(query, skip, limit)
            return result.rooms if result.rooms else None

        return offset_based_pagination(search_rooms_one_page, chunk_size, max_number)

    @retry
    async def get_room_info(self, room_id: str) -> V3RoomDetail:
        """Get information about a particular room.
        Wraps the `Room Info V3 <https://developers.symphony.com/restapi/reference#room-info-v3>`_ endpoint.

        :param room_id: the id of the room.
        :return: the room details.
        """
        return await self._streams_api.v3_room_id_info_get(id=room_id,
                                                           session_token=await self._auth_session.session_token)

    @retry
    async def set_room_active(self, room_id: str, active: bool) -> RoomDetail:
        """Deactivates or reactivates a chatroom. At creation time, the chatroom is activated by default.
        Wraps the `De/Reactivate Room <https://developers.symphony.com/restapi/reference#de-or-re-activate-room>`_
        endpoint.

        :param room_id: the id of the room to be deactivated or reactivated.
        :param active: the new active status (True to reactivate, false to deactivate).
        :return: the details of the updated room.
        """
        return await self._streams_api.v1_room_id_set_active_post(id=room_id, active=active,
                                                                  session_token=await self._auth_session.session_token)

    @retry
    async def update_room(self, room_id: str, room_attributes: V3RoomAttributes) -> V3RoomDetail:
        """Updates the attributes of an existing chatroom.
        Wraps the `Update Room V3 <https://developers.symphony.com/restapi/reference#update-room-v3>`_ endpoint.

        :param room_id: the id of the room to be updated.
        :param room_attributes: the attributes of the room to be updated.
        :return: the details of the updated room.
        """
        return await self._streams_api.v3_room_id_update_post(id=room_id, payload=room_attributes,
                                                              session_token=await self._auth_session.session_token)

    @retry
    async def create_im_admin(self, user_ids: [int]) -> Stream:
        """Create a new single or multi party instant message conversation.
        At least two user IDs must be provided or an error response will be sent.
        The caller is not included in the members of the created chat.
        Duplicate users will be included in the membership of the chat but the duplication will be silently ignored.
        If there is an existing IM conversation with the same set of participants then the id of that existing stream
        will be returned.

        Wraps the
        `Create IM or MIM Non-inclusive <https://developers.symphony.com/restapi/reference#create-im-or-mim-admin>`_
        endpoint.

        :param user_ids: the list of user IDs to be put as participants. At least two user IDs must be provided.
        :return: the created IM or MIM.
        """
        return await self._streams_api.v1_admin_im_create_post(uid_list=UserIdList(value=user_ids),
                                                               session_token=await self._auth_session.session_token)

    @retry
    async def set_room_active_admin(self, room_id: str, active: bool) -> RoomDetail:
        """Deactivates or reactivates a chatroom via AC Portal.

        :param room_id: the id of the room to be deactivated or reactivated.
        :param active: the new active status (True to reactivate, false to deactivate).
        :return: the details of the updated room.
        """
        return await self._streams_api.v1_admin_room_id_set_active_post(
            id=room_id, active=active,
            session_token=await self._auth_session.session_token)

    @retry
    async def list_streams_admin(self, stream_filter: V2AdminStreamFilter, skip: int = 0,
                                 limit: int = 50) -> V2AdminStreamList:
        """Retrieves all the streams across the enterprise.
        Wraps the `List Streams for Enterprise V2
        <https://developers.symphony.com/restapi/reference#list-streams-for-enterprise-v2>`_ endpoint.

        :param stream_filter: the stream searching filter.
        :param skip: the number of streams to skip.
        :param limit: the maximum number of streams to retrieve. Must be a positive integer less or equal than 100.
        :return: the list of streams matching the search criteria.
        """
        return await self._streams_api.v2_admin_streams_list_post(filter=stream_filter, skip=skip, limit=limit,
                                                                  session_token=await self._auth_session.session_token)

    async def list_all_streams_admin(self, stream_filter: V2AdminStreamFilter, chunk_size=50, max_number=None) \
            -> AsyncGenerator[V2AdminStreamInfo, None]:
        """Retrieves all the streams across the enterprise.
        Wraps the `List Streams for Enterprise V2
        <https://developers.symphony.com/restapi/reference#list-streams-for-enterprise-v2>`_ endpoint.

        :param stream_filter: the stream searching filter.
        :param chunk_size: the maximum number of elements to retrieve in one underlying HTTP call.
        :param max_number: the total maximum number of elements to retrieve.
        :return: an asynchronous generator of streams matching the search criteria.
        """

        async def list_streams_admin_one_page(skip, limit):
            result = await self.list_streams_admin(stream_filter, skip, limit)
            return result.streams.value if result.streams else None

        return offset_based_pagination(list_streams_admin_one_page, chunk_size, max_number)

    @retry
    async def list_stream_members(self, stream_id: str, skip: int = 0, limit: int = 100) -> V2MembershipList:
        """List the current members of an existing stream. The stream can be of type IM, MIM, or ROOM.
        Wraps the `Stream Members <https://developers.symphony.com/restapi/reference#stream-members>`_ endpoint.

        :param stream_id: the ID of the stream.
        :param skip: the number of members to skip.
        :param limit: the maximum number of members to retrieve. Must be less or equal than 1000.
        :return: the list of stream members.
        """
        return await self._streams_api.v1_admin_stream_id_membership_list_get(
            id=stream_id, skip=skip, limit=limit,
            session_token=await self._auth_session.session_token)

    async def list_all_stream_members(self, stream_id: str, chunk_size: int = 50, max_number=None) \
            -> AsyncGenerator[V2MemberInfo, None]:
        """List the current members of an existing stream. The stream can be of type IM, MIM, or ROOM.
        Wraps the `Stream Members <https://developers.symphony.com/restapi/reference#stream-members>`_ endpoint.

        :param stream_id: the ID of the stream.
        :param chunk_size: the maximum number of elements to retrieve in one underlying HTTP call.
        :param max_number: the total maximum number of elements to retrieve.
        :return: an asynchronous generator of the stream members.
        """
        async def list_stream_members_one_page(skip, limit):
            members = await self.list_stream_members(stream_id, skip, limit)
            return members.members.value if members.members else None

        return offset_based_pagination(list_stream_members_one_page, chunk_size, max_number)

    @retry
    async def list_room_members(self, room_id: str) -> MembershipList:
        """Lists the current members of an existing room.
        Wraps the `Room Members <https://developers.symphony.com/restapi/reference#room-members>`_ endpoint.

        :param room_id: the ID of the room.
        :return: the list of room members.
        """
        return await self._room_membership_api.v2_room_id_membership_list_get(
            id=room_id,
            session_token=await self._auth_session.session_token)



from enum import Enum, auto
from typing import List

from symphony.bdk.core.auth.auth_session import AuthSession
from symphony.bdk.core.config.model.bdk_retry_config import BdkRetryConfig
from symphony.bdk.core.retry import retry
from symphony.bdk.gen.pod_api.presence_api import PresenceApi
from symphony.bdk.gen.pod_model.v2_presence import V2Presence
from symphony.bdk.gen.pod_model.v2_presence_status import V2PresenceStatus
from symphony.bdk.gen.pod_model.v2_user_presence import V2UserPresence


class PresenceStatus(Enum):
    """The list of all possible values for the presence status.
    Set Presence <https://developers.symphony.com/restapi/reference#set-presence>`_
    """

    AVAILABLE = auto()
    BUSY = auto()
    AWAY = auto()
    ON_THE_PHONE = auto()
    BE_RIGHT_BACK = auto()
    IN_A_MEETING = auto()
    OUT_OF_OFFICE = auto()
    OFF_WORK = auto()


class OboPresenceService:
    """Service class exposing OBO-enabled endpoints to manage user presence information.

    This service is used for retrieving information about the presence of the OBO user or a specified user or
    all users in the pod, and perform some actions related to the user presence information like:

    * Set Presence to calling user
    * Set Presence to a specified user
    * Create a presence feed
    * Read a created presence feed
    * Delete a created presence feed
    """

    def __init__(self, presence_api: PresenceApi, auth_session: AuthSession, retry_config: BdkRetryConfig):
        self._presence_api = presence_api
        self._auth_session = auth_session
        self._retry_config = retry_config

    @retry
    async def get_presence(self) -> V2Presence:
        """ Get the online status (presence info) of the calling user.
        See: `Get Presence <https://developers.symphony.com/restapi/reference#get-presence>`_.

        :return: Presence info of the calling user.
        """
        return await self._presence_api.v2_user_presence_get(session_token=await self._auth_session.session_token)

    async def get_all_presence(self, last_user_id: int, limit: int) -> List[V2Presence]:
        """ Get the presence info of all users in a pod.
        See: `Get All Presence <https://developers.symphony.com/restapi/reference#get-all-presence>`_.

        :param last_user_id: Last user ID retrieved, used for paging. If provided, results skip users with IDs less
          than this parameter.
        :param limit: Maximum number of records to return. The maximum supported value is 5000.
        :return: Presence info of the looked up user.
        """
        presence_list = await self._presence_api.v2_users_presence_get(
            session_token=await self._auth_session.session_token,
            last_user_id=last_user_id,
            limit=limit)
        return presence_list.value

    @retry
    async def get_user_presence(self, user_id: int, local: bool) -> V2Presence:
        """ Get the presence info of a specified user.
        See: `Get User Presence <https://developers.symphony.com/restapi/reference#user-presence-v3>`_.

        :param user_id: User Id
        :param local: If true then Perform a local query and set the presence to OFFLINE for  users who are not local to
          the calling user’s pod. If false or absent then query the presence of all local  and external users who are
          connected to the calling user.
        :return: Presence info of the looked up user.
        """
        return await self._presence_api.v3_user_uid_presence_get(uid=user_id,
                                                                 session_token=await self._auth_session.session_token,
                                                                 local=local)

    @retry
    async def external_presence_interest(self, user_ids: List[int]):
        """ Register interest in a list of external users to get their presence info.
        See: `External Presence Interest
        <https://developers.symphony.com/restapi/reference#register-user-presence-interest>`_.

        :param user_ids: List of user ids to be registered.
        """
        await self._presence_api.v1_user_presence_register_post(session_token=await self._auth_session.session_token,
                                                                uid_list=user_ids)

    @retry
    async def set_presence(self, status: PresenceStatus, soft: bool) -> V2Presence:
        """ Set the presence info of the calling user.
        See: `Set Presence <https://developers.symphony.com/restapi/reference#set-presence>`_.

        :param status: The new presence state for the user.
          Possible values are AVAILABLE, BUSY, AWAY, ON_THE_PHONE, BE_RIGHT_BACK, IN_A_MEETING, OUT_OF_OFFICE, OFF_WORK.
        :param soft: If true, the user's current status is taken into consideration. If the user is currently OFFLINE,
          the user's presence will still be OFFLINE, but the new presence will take effect when the user comes online.
          If the user is currently online, the user's activity state will be applied to the presence if applicable.
          (e.g. if you are setting their presence to AVAILABLE, but the user is currently idle, their status will be
          represented as AWAY)
        :return: Presence info of the calling user.
        """
        presence_status: V2PresenceStatus = V2PresenceStatus(category=status.name)
        return await self._presence_api.v2_user_presence_post(session_token=await self._auth_session.session_token,
                                                              presence=presence_status,
                                                              soft=soft)

    @retry
    async def create_presence_feed(self) -> str:
        """ Creates a new stream capturing online status changes ("presence feed") for the company (pod) and returns
        the ID of the new feed. The feed will return the presence of users whose presence status has changed since it
        was last read.
        See: `Create Presence Feed <https://developers.symphony.com/restapi/reference#create-presence-feed>`_.

        :return: Presence feed Id
        """
        string_id = await self._presence_api.v1_presence_feed_create_post(
            session_token=await self._auth_session.session_token)
        return string_id.id

    @retry
    async def read_presence_feed(self, feed_id: str) -> List[V2Presence]:
        """ Reads the specified presence feed that was created.
        The feed returned includes the user presence statuses that have changed since they were last read.
        See: `Read Presence Feed <https://developers.symphony.com/restapi/reference#read-presence-feed>`_.

        :param feed_id: The presence feed id to be read.
        :return: The list of user presences has changed since the last presence read.
        """
        presence_list = await self._presence_api.v1_presence_feed_feed_id_read_get(
            session_token=await self._auth_session.session_token,
            feed_id=feed_id)
        return presence_list.value

    @retry
    async def delete_presence_feed(self, feed_id: str) -> str:
        """ Delete the specified presence feed that was created.
        See: `Delete Presence Feed <https://developers.symphony.com/restapi/reference#delete-presence-feed>`_.

        :param feed_id: The presence feed id to be deleted.
        :return: The id of the deleted presence feed.
        """
        string_id = await self._presence_api.v1_presence_feed_feed_id_delete_post(
            session_token=await self._auth_session.session_token, feed_id=feed_id)
        return string_id.id

    @retry
    async def set_user_presence(self, user_id: int, status: PresenceStatus, soft: bool) -> V2Presence:
        """ Set the presence state of a another user.
        See: `Set Other User's Presence - Admin V3
        <https://developers.symphony.com/restapi/reference#set-user-presence>`_.

        :param user_id: The id of the specified user.
        :param status: Presence state to set.
          Possible values are AVAILABLE, BUSY, AWAY, ON_THE_PHONE, BE_RIGHT_BACK, IN_A_MEETING, OUT_OF_OFFICE, OFF_WORK.
        :param soft: If true, the user's current status is taken into consideration. If the user is currently OFFLINE,
          the user's presence will still be OFFLINE, but the new presence will take effect when the user comes online.
          If the user is currently online, the user's activity state will be applied to the presence if applicable.
          (e.g. if you are setting their presence to AVAILABLE, but the user is currently idle, their status will be
          represented as AWAY)
        :return: The presence info of the specified user.
        """
        user_presence: V2UserPresence = V2UserPresence(category=status.name, user_id=user_id)
        return await self._presence_api.v3_user_presence_post(session_token=await self._auth_session.session_token,
                                                              presence=user_presence,
                                                              soft=soft)


class PresenceService(OboPresenceService):
    """Service class for managing user presence information.

    This service is used for retrieving information about the presence of the calling user or a specified user or
    all users in the pod, and perform some actions related to the user presence information like:

    * Set Presence to calling user
    * Set Presence to a specified user
    * Create a presence feed
    * Read a created presence feed
    * Delete a created presence feed
    """



import logging
from typing import Callable
from functools import wraps

from tenacity import stop_after_attempt, wait_exponential, before_sleep_log

from symphony.bdk.core.config.model.bdk_retry_config import BdkRetryConfig
from symphony.bdk.core.retry.strategy import refresh_session_if_unauthorized

from ._asyncio import AsyncRetrying


def retry(*dargs, **dkw):
    """A decorator that provides a mechanism to to retry failed requests
    Passed retry configuration arguments will override the default configuration defined in :py:meth:`decorator_f`
    If no _retry_config attribute is present in the decorated function instance, an AttributeError is raised.

    :param dargs: positional arguments passed to be added or to override the default configuration
    :param dkw: keyword arguments passed to be added or to override the default configuration
    """
    # support both @retry and @retry() as valid syntax
    if len(dargs) == 1 and callable(dargs[0]):
        return retry()(dargs[0])

    def retry_decorator(fun: Callable):
        @wraps(fun)
        def decorator_f(self, *args, **kwargs):
            """Fetches a BdkRetryConfiguration object from the instance of the called function and constructs the
            arguments for the AsyncRetrying object.
            """
            default_kwargs = {}
            retry_config: BdkRetryConfig = getattr(self, '_retry_config')
            logger = logging.getLogger(self.__module__)
            _before_sleep = before_sleep_log(logger, logging.INFO)
            default_kwargs.update(dict(before_sleep=_before_sleep))
            if retry_config is not None:
                config_kwargs = dict(retry=refresh_session_if_unauthorized,
                                     wait=wait_exponential(multiplier=retry_config.multiplier,
                                                           min=retry_config.initial_interval.total_seconds(),
                                                           max=retry_config.max_interval.total_seconds()),
                                     stop=stop_after_attempt(retry_config.max_attempts),
                                     reraise=True)
                default_kwargs.update(config_kwargs)
            # update default arguments by the ones passed as parameters
            default_kwargs.update(**dkw)
            return AsyncRetrying(*dargs, **default_kwargs).wraps(fun)(self, *args, **kwargs)

        return decorator_f

    return retry_decorator



import enum


class ConnectionStatus(enum.Enum):
    """The list of all possible values for the request listing connection status.
    See: `List Connections <https://developers.symphony.com/restapi/reference#list-connections>`_
    """
    ALL = "ALL"
    PENDING_INCOMING = "PENDING_INCOMING"
    PENDING_OUTGOING = "PENDING_OUTGOING"
    ACCEPTED = "ACCEPTED"
    REJECTED = "REJECTED"



"""Module containing SymphonyBdk class which acts as en entry point for all BDK services.
"""
import functools
import logging

from symphony.bdk.core.activity.registry import ActivityRegistry
from symphony.bdk.core.auth.auth_session import AuthSession, OboAuthSession
from symphony.bdk.core.auth.authenticator_factory import AuthenticatorFactory
from symphony.bdk.core.auth.exception import AuthInitializationError
from symphony.bdk.core.auth.ext_app_authenticator import ExtensionAppAuthenticator
from symphony.bdk.core.client.api_client_factory import ApiClientFactory
from symphony.bdk.core.config.exception import BotNotConfiguredError, BdkConfigError
from symphony.bdk.core.service.application.application_service import ApplicationService
from symphony.bdk.core.service.connection.connection_service import ConnectionService
from symphony.bdk.core.service.datafeed.abstract_datafeed_loop import AbstractDatafeedLoop
from symphony.bdk.core.service.health.health_service import HealthService
from symphony.bdk.core.service.message.message_service import MessageService
from symphony.bdk.core.service.obo_services import OboServices
from symphony.bdk.core.service.presence.presence_service import PresenceService
from symphony.bdk.core.service.session.session_service import SessionService
from symphony.bdk.core.service.signal.signal_service import SignalService
from symphony.bdk.core.service.stream.stream_service import StreamService
from symphony.bdk.core.service.user.user_service import UserService
from symphony.bdk.core.service_factory import ServiceFactory

logger = logging.getLogger(__name__)


def bot_service(func):
    """Decorator to check if a bot service account is configured before making the actual function call.

    :param func: the decorated function.
    :return: the value returned by the decorated function with the passed arguments.
    :raise: BotNotConfiguredError if the bot service account is not configured.
    """

    @functools.wraps(func)
    def check_if_bot_configured_and_call_function(*args):
        symphony_bdk = args[0]
        if not symphony_bdk._config.bot.is_authentication_configured():
            raise BotNotConfiguredError("Error calling bot service ")
        if not symphony_bdk._config.is_bot_configured():
            raise BotNotConfiguredError("Bot username field is not configured correctly.")
        return func(*args)

    return check_if_bot_configured_and_call_function


def app_service(func):
    """Decorator to check if an app is configured before making the actual function call.

    :param func: the decorated function.
    :return: the value returned by the decorated function with the passed arguments.
    :raise: BdkConfigError if the app is not configured.
    """
    @functools.wraps(func)
    def check_if_app_configured_and_call_function(*args, **kwds):
        symphony_bdk = args[0]
        if not symphony_bdk._config.is_app_configured():
            raise BdkConfigError("App appId field is not configured correctly.")
        return func(*args, **kwds)

    return check_if_app_configured_and_call_function


class SymphonyBdk:
    """BDK entry point
    """

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close_clients()

    def __init__(self, config):
        self._config = config
        self._api_client_factory = ApiClientFactory(config)
        self._pod_client = self._api_client_factory.get_pod_client()
        self._agent_client = self._api_client_factory.get_agent_client()
        self._authenticator_factory = AuthenticatorFactory(config, api_client_factory=self._api_client_factory)

        self._bot_session = None
        self._ext_app_authenticator = None
        self._service_factory = None
        self._user_service = None
        self._message_service = None
        self._connection_service = None
        self._stream_service = None
        self._application_service = None
        self._signal_service = None
        self._session_service = None
        self._datafeed_loop = None
        self._health_service = None
        self._presence_service = None
        self._activity_registry = None

        if self._config.bot.is_authentication_configured():
            self._initialize_bot_services()
        else:
            logging.info("Bot (service account) credentials have not been configured."
                         "You can however use services in OBO mode if app authentication is configured.")

    def _initialize_bot_services(self):
        self._bot_session = AuthSession(self._authenticator_factory.get_bot_authenticator())
        self._service_factory = ServiceFactory(self._api_client_factory, self._bot_session, self._config)
        self._user_service = self._service_factory.get_user_service()
        self._message_service = self._service_factory.get_message_service()
        self._connection_service = self._service_factory.get_connection_service()
        self._stream_service = self._service_factory.get_stream_service()
        self._application_service = self._service_factory.get_application_service()
        self._signal_service = self._service_factory.get_signal_service()
        self._session_service = self._service_factory.get_session_service()
        self._datafeed_loop = self._service_factory.get_datafeed_loop()
        self._health_service = self._service_factory.get_health_service()
        self._presence_service = self._service_factory.get_presence_service()
        # creates ActivityRegistry that subscribes to DF Loop events
        self._activity_registry = ActivityRegistry(self._session_service)
        self._datafeed_loop.subscribe(self._activity_registry)

    @bot_service
    def bot_session(self) -> AuthSession:
        """Get the Bot authentication session. If the bot is not authenticated yet, perform the authentication for a new
        session.

        :return: The bot authentication session.
        """
        return self._bot_session

    @app_service
    def app_authenticator(self) -> ExtensionAppAuthenticator:
        """Get the extension app authenticator.

        :return: a new instance of :class:`symphony.bdk.core.auth.ext_app_authenticator.ExtensionAppAuthenticator`
        """
        if self._ext_app_authenticator is None:
            self._ext_app_authenticator = self._authenticator_factory.get_extension_app_authenticator()
        return self._ext_app_authenticator

    @app_service
    def obo(self, user_id: int = None, username: str = None) -> OboAuthSession:
        """Get the Obo authentication session.

        :return: The obo authentication session
        """
        if user_id is not None:
            return self._authenticator_factory.get_obo_authenticator().authenticate_by_user_id(user_id)
        if username is not None:
            return self._authenticator_factory.get_obo_authenticator().authenticate_by_username(username)
        raise AuthInitializationError("At least user_id or username should be given to OBO authenticate the "
                                      "extension app")

    @app_service
    def obo_services(self, obo_session: OboAuthSession) -> OboServices:
        """Return the entry point of all OBO-enabled services and endpoints.

        :param obo_session: the obo_session to use.
        :return: a new OboServices instance.
        """
        return OboServices(self._config, obo_session)

    @bot_service
    def messages(self) -> MessageService:
        """Get the MessageService from the BDK entry point.

        :return: The MessageService instance.
        """
        return self._message_service

    @bot_service
    def streams(self) -> StreamService:
        """Get the StreamService from the BDK entry point.

        :return: The StreamService instance.
        """
        return self._stream_service

    @bot_service
    def datafeed(self) -> AbstractDatafeedLoop:
        """Get the Datafeed loop from the BDK entry point.

        :return: The Datafeed Loop instance.
        """
        return self._datafeed_loop

    @bot_service
    def users(self) -> UserService:
        """Get the UserService from the BDK entry point.

        :return: The UserService instance.
        """
        return self._user_service

    @bot_service
    def connections(self) -> ConnectionService:
        """Get the ConnectionService from the BDK entry point.

        :return: The ConnectionService instance.
        """
        return self._connection_service

    @bot_service
    def applications(self) -> ApplicationService:
        """Get the ApplicationService from the BDK entry point.

        :return: The ApplicationService instance.
        """
        return self._application_service

    @bot_service
    def signals(self) -> SignalService:
        """Get the SignalService from the BDK entry point.

        :return: The SignalService instance.
        """
        return self._signal_service

    @bot_service
    def sessions(self) -> SessionService:
        """Get the SessionService from the BDK entry point.

        :return: The SessionService instance.
        """
        return self._session_service

    @bot_service
    def health(self) -> HealthService:
        """Get the HealthService from the BDK entry point.

        :return: The HealthService instance.
        """
        return self._health_service

    @bot_service
    def presence(self) -> PresenceService:
        """Get the PresenceService from the BDK entry point.

        :return: The PresenceService instance.
        """
        return self._presence_service

    @bot_service
    def activities(self) -> ActivityRegistry:
        """Get the :class:`ActivityRegistry` from the BDK entry point.

        :return: The :class:`ActivityRegistry` instance.

        """
        return self._activity_registry

    async def close_clients(self):
        """Close all the existing api clients created by the api client factory.
        """
        await self._api_client_factory.close_clients()



import logging

from symphony.bdk.core.activity.api import AbstractActivity, ActivityContext
from symphony.bdk.gen.agent_model.v4_initiator import V4Initiator
from symphony.bdk.gen.agent_model.v4_symphony_elements_action import V4SymphonyElementsAction

logger = logging.getLogger(__name__)


class FormReplyContext(ActivityContext[V4SymphonyElementsAction]):
    """
    Default implementation of the :py:class`ActivityContext` handled by the :py:class:`FormReplyActivity`
    """

    def __init__(self, initiator: V4Initiator, source_event: V4SymphonyElementsAction):
        self._form_id = source_event.form_id
        self._form_values = source_event.form_values
        super().__init__(initiator, source_event)

    @property
    def form_id(self) -> str:
        return self._form_id

    @property
    def form_values(self) -> dict:
        return self._form_values

    def get_form_value(self, field_name: str) -> str:
        """Get the value of specified form field

        :param field_name: the form field name
        :return: the form field value if it is present, None otherwise
        """
        return self.form_values.get(field_name)


class FormReplyActivity(AbstractActivity[FormReplyContext]):
    """
    A form reply activity corresponds to an Elements form submission.
    """

    def matches(self, context: FormReplyContext) -> bool:
        pass

    async def on_activity(self, context: FormReplyContext):
        pass



from aiohttp import ClientConnectorError
from tenacity import RetryCallState

from symphony.bdk.core.auth.exception import AuthUnauthorizedError
from symphony.bdk.gen import ApiException


def is_unauthorized(exception: Exception) -> bool:
    return isinstance(exception, ApiException) and exception.status == 401


def is_client_error(exception: Exception) -> bool:
    return isinstance(exception, ApiException) and exception.status == 400


def is_client_timeout_error(exception: Exception):
    """Checks if the exception is a :py:class:`ClientConnectorError` with a :py:class:`TimeoutError` as cause

    :param exception: The exception to be checked
    :return: True if checks the predicate, False otherwise
    """
    return isinstance(exception, ClientConnectorError) and isinstance(exception.__cause__, TimeoutError)


def can_authentication_be_retried(exception: Exception) -> bool:
    """Checks if the exception is a network issue or internal or retry error
    i.e. is an :py:class:`ApiException` with status code 429 or greater or equal to 500.
    or the exception is either a :py:class:`TimeoutError`  (e.g. network issues)

    :param exception: The exception to be checked
    :return: True if the exception is an :py:class:`ApiException` with status 500, 429 or or is a client timeout error
             False otherwise
    """
    if isinstance(exception, ApiException):
        return exception.status >= 500 or exception.status == 429
    return is_client_timeout_error(exception)


def _is_minor_error(exception: ApiException):
    return exception.status >= 500 or exception.status == 401 or exception.status == 429


def is_network_or_minor_error(exception: Exception) -> bool:
    """Checks if the exception is a network issue or an :py:class:`ApiException` minor error (internal +
    unauthorized + retry). This is the default function used to check if a given exception should lead to a retry.

    :param exception: The exception to be checked
    :return: True if the exception is an :py:class:`ApiException` with status 500, 401, 429 or is a client timeout error
             False otherwise
    """
    if isinstance(exception, ApiException):
        return _is_minor_error(exception)
    return is_client_timeout_error(exception)


def is_network_or_minor_error_or_client(exception: Exception) -> bool:
    """Checks if the exception is a network issue or an :py:class:`ApiException` minor error or a client error

    :param exception: The exception to be checked
    :return: True if the exception is an :py:class:`ApiException` with status 500, 401, 429, 400
             or is a client timeout error False otherwise
    """
    if isinstance(exception, ApiException):
        return _is_minor_error(exception) or exception.status == 400
    return is_client_timeout_error(exception)


def authentication_retry(retry_state: RetryCallState):
    """Authentication retry strategy

    Retry if the raised exception verifies the can_authentication_be_retried predicate
    Raise an :py:class`AuthUnauthorizedException` if an :py:class`ApiException` with status 401 is thrown
    Raise all other exception
    """
    if retry_state.outcome.failed:
        exception = retry_state.outcome.exception()
        if can_authentication_be_retried(exception):
            return True
        if is_unauthorized(exception):
            unauthorized_message = "Service account is not authorized to authenticate. Check if credentials are valid. "
            raise AuthUnauthorizedError(unauthorized_message, exception) from exception
        raise exception
    return False


async def refresh_session_if_unauthorized(retry_state: RetryCallState):
    """Refreshing AuthSession tokens retry strategy

    Retry if the raised exception verifies the is_network_or_minor_error predicate
    If this exception is a 401 unauthorized error, we refresh the current AuthSession tokens
    """
    if retry_state.outcome.failed:
        exception = retry_state.outcome.exception()
        if is_network_or_minor_error(exception):
            if is_unauthorized(exception):
                service_auth_session = retry_state.args[0]._auth_session
                await service_auth_session.refresh()
            return True
    return False


async def read_datafeed_retry(retry_state: RetryCallState):
    """Read datafeed retry strategy

    Retry if the raised exception verifies the is_network_or_minor_error_or_client predicate
    If this exception is a 400 client error, we recreate the datafeed
    If this exception is a 401 unauthorized error, we refresh the current AuthSession tokens
    """
    if retry_state.outcome.failed:
        exception = retry_state.outcome.exception()
        if is_network_or_minor_error_or_client(exception):
            if is_client_error(exception):
                datafeed_service = retry_state.args[0]  # datafeed_service is an AbstractDataFeedLoop instance
                await datafeed_service.recreate_datafeed()
            elif is_unauthorized(exception):
                service_auth_session = retry_state.args[0]._auth_session
                await service_auth_session.refresh()
            return True
        raise exception
    return False



from typing import AsyncGenerator

from symphony.bdk.core.auth.auth_session import AuthSession
from symphony.bdk.core.config.model.bdk_retry_config import BdkRetryConfig
from symphony.bdk.core.retry import retry
from symphony.bdk.core.service.pagination import offset_based_pagination
from symphony.bdk.gen.agent_api.signals_api import SignalsApi
from symphony.bdk.gen.agent_model.base_signal import BaseSignal
from symphony.bdk.gen.agent_model.channel_subscriber import ChannelSubscriber
from symphony.bdk.gen.agent_model.channel_subscriber_response import ChannelSubscriberResponse
from symphony.bdk.gen.agent_model.channel_subscription_response import ChannelSubscriptionResponse
from symphony.bdk.gen.agent_model.signal import Signal
from symphony.bdk.gen.agent_model.signal_list import SignalList


class OboSignalService:
    """Service class exposing OBO-enabled endpoints to manage signal information.

    This service is used for listing signals related to the OBO user, get information of a specified signal
    or perform some actions related to the signal like:

    * List signals
    * Get a signal
    * Create a signal
    * Update a signal
    * Delete a signal
    * Subscribe or unsubscribe a signal
    """

    def __init__(self, signals_api: SignalsApi, auth_session: AuthSession, retry_config: BdkRetryConfig):
        self._signals_api = signals_api
        self._auth_session = auth_session
        self._retry_config = retry_config

    @retry
    async def list_signals(self, skip: int = 0, limit: int = 50) -> SignalList:
        """Lists signals on behalf of the user. The response includes signals that the user has created and
        public signals to which they have subscribed.

        See: 'List signals <https://developers.symphony.com/restapi/reference#list-signals>'_

        :param skip: The number of signals to skip.
        :param limit: Maximum number of signals to return. Default is 50, maximum value is 500.
        :return: List of signals found.
        """

        return await self._signals_api.v1_signals_list_get(
            skip=skip, limit=limit, session_token=await self._auth_session.session_token,
            key_manager_token=await self._auth_session.key_manager_token)

    async def list_all_signals(self, chunk_size: int = 50, max_number: int = None) -> AsyncGenerator[Signal, None]:
        """Lists all signals on behalf of the user. The response includes signals that the user has created and
        public signals to which they have subscribed.

        See: 'List signals <https://developers.symphony.com/restapi/reference#list-signals>'_

        :param chunk_size: the maximum number of elements to retrieve in one underlying HTTP call
        :param max_number: the total maximum number of elements to retrieve
        :return: an asynchronous generator of found signals
        """

        async def list_signals_one_page(skip, limit):
            result = await self.list_signals(skip, limit)
            return result.value if result else None

        return offset_based_pagination(list_signals_one_page, chunk_size, max_number)

    @retry
    async def get_signal(self, signal_id: str) -> Signal:
        """ Gets details about the specified signal.

        See: 'Get signal <https://developers.symphony.com/restapi/reference#get-signal>'_

        :param signal_id: Id of the signal to display.
        :return: The signal found.
        """

        return await self._signals_api.v1_signals_id_get_get(
            id=signal_id, session_token=await self._auth_session.session_token,
            key_manager_token=await self._auth_session.key_manager_token)

    @retry
    async def create_signal(self, signal: BaseSignal) -> Signal:
        """ Creates a new Signal.

        See: 'Create signal <https://developers.symphony.com/restapi/reference#create-signal>'_

        :param signal: The new Signal object to be created.
        :return: The signal created.
        """

        return await self._signals_api.v1_signals_create_post(
            signal=signal, session_token=await self._auth_session.session_token,
            key_manager_token=await self._auth_session.key_manager_token)

    @retry
    async def update_signal(self, signal_id: str, signal: BaseSignal) -> Signal:
        """ Updates an existing Signal.

        See: 'Update signal <https://developers.symphony.com/restapi/reference#update-signal>'_

        :param signal_id: The Id of the signal to be modified.
        :param signal: The Signal object to be updated.
        :return: The updated signal.
        """

        return await self._signals_api.v1_signals_id_update_post(
            id=signal_id, signal=signal, session_token=await self._auth_session.session_token,
            key_manager_token=await self._auth_session.key_manager_token)

    @retry
    async def delete_signal(self, signal_id: str) -> None:
        """ Deletes an existing Signal.

        See: 'Delete signal <https://developers.symphony.com/restapi/reference#delete-signal>'_

        :param signal_id: The Id of the existing signal to be deleted.
        """

        await self._signals_api.v1_signals_id_delete_post(id=signal_id,
                                                          session_token=await self._auth_session.session_token,
                                                          key_manager_token=await self._auth_session.key_manager_token)

    @retry
    async def subscribe_users_to_signal(self, signal_id: str, pushed: bool,
                                        user_ids: [int]) -> ChannelSubscriptionResponse:
        """ Subscribe an array of users to a Signal.

        See: 'Subscribe signal <https://developers.symphony.com/restapi/reference#subscribe-signal>'_

        :param signal_id: The Id of the signal to be subscribed.
        :param pushed: Prevents the user from unsubscribing from the Signal
        :param user_ids: An array of User Ids to subscribe to the Signal
        :return: Result of the bulk subscriptions
        """

        return await self._signals_api.v1_signals_id_subscribe_post(
            id=signal_id, pushed=pushed, users=user_ids, session_token=await self._auth_session.session_token,
            key_manager_token=await self._auth_session.key_manager_token)

    @retry
    async def unsubscribe_users_to_signal(self, signal_id: str, user_ids: [int]) -> ChannelSubscriptionResponse:
        """ Unsubscribes an array of users from the specified Signal.

        See: 'Unsubscribe signal <https://developers.symphony.com/restapi/reference#unsubscribe-signal>'_

        :param signal_id: The Id of the signal to be subscribed.
        :param user_ids: An array of User Ids to subscribe to the Signal
        :return: Result of the bulk unsubscriptions
        """

        return await self._signals_api.v1_signals_id_unsubscribe_post(
            id=signal_id, users=user_ids, session_token=await self._auth_session.session_token,
            key_manager_token=await self._auth_session.key_manager_token)

    @retry
    async def list_subscribers(self, signal_id: str, skip: int = 0, limit: int = 50) -> ChannelSubscriberResponse:
        """Gets the subscribers for the specified signal.

        See: 'Subscribers <https://developers.symphony.com/restapi/reference#subscribers>'_

        :param signal_id: The Id of the signal.
        :param skip: The number of results to skip.
        :param limit: The maximum number of subscribers to return. The maximum value accepted for this parameter is 100
          and the default value is 50.
        :return: The list of users subscribed to the signal.
        """

        return await self._signals_api.v1_signals_id_subscribers_get(
            id=signal_id, skip=skip, limit=limit, session_token=await self._auth_session.session_token,
            key_manager_token=await self._auth_session.key_manager_token)

    async def list_all_subscribers(self, signal_id: str, chunk_size: int = 50, max_number: int = None) \
            -> AsyncGenerator[ChannelSubscriber, None]:
        """Gets all the subscribers for the specified signal.

        See: 'Subscribers <https://developers.symphony.com/restapi/reference#subscribers>'_

        :param signal_id: the Id of the signal.
        :param chunk_size: the maximum number of elements to retrieve in one underlying HTTP call.
        :param max_number: the total maximum number of elements to retrieve.
        :return: an asynchronous generator returning all users subscribed to the signal.
        """

        async def list_subscribers_one_page(skip, limit):
            result = await self.list_subscribers(signal_id, skip, limit)
            return result.data if result else None

        return offset_based_pagination(list_subscribers_one_page, chunk_size, max_number)


class SignalService(OboSignalService):
    """Service class for managing signal information.
    This service is used for listing signals related to the user, get information of a specified signal
    or perform some actions related to the signal like:

    * List signals
    * Get a signal
    * Create a signal
    * Update a signal
    * Delete a signal
    * Subscribe or unsubscribe a signal
    """



import logging

from symphony.bdk.core.activity.api import AbstractActivity, ActivityContext
from symphony.bdk.core.activity.exception import FatalActivityExecutionException
from symphony.bdk.core.service.exception import MessageParserError
from symphony.bdk.core.service.message.message_parser import get_text_content_from_message
from symphony.bdk.gen.agent_model.v4_initiator import V4Initiator
from symphony.bdk.gen.agent_model.v4_message_sent import V4MessageSent

logger = logging.getLogger(__name__)


class CommandContext(ActivityContext[V4MessageSent]):
    """Default implementation of the :py:class:`ActivityContext` handled by the :py:class:`CommandActivity`.
    """

    def __init__(self, initiator: V4Initiator, source_event: V4MessageSent, bot_display_name: str):
        self._message_id = source_event.message.message_id
        self._stream_id = source_event.message.stream.stream_id
        self._bot_display_name = bot_display_name
        try:
            self._text_content = get_text_content_from_message(source_event.message)
        except MessageParserError as exc:
            raise FatalActivityExecutionException("Unable to parse presentationML") from exc
        super().__init__(initiator, source_event)

    @property
    def message_id(self):
        return self._message_id

    @property
    def stream_id(self):
        return self._stream_id

    @property
    def text_content(self) -> str:
        return self._text_content

    @property
    def bot_display_name(self) -> str:
        return self._bot_display_name


class CommandActivity(AbstractActivity[CommandContext]):
    """A command activity corresponds to any message sent in a chat where the bot is part of.
    """

    def matches(self, context: CommandContext) -> bool:
        pass

    async def on_activity(self, context: CommandContext):
        pass


class SlashCommandActivity(CommandActivity):
    """A slash command is a particular CommandActivity with three parameters:
      - the command name
      - if we should mention the bot
      - the callback function

    The Slash command is triggered if we receive a MESSAGESENT event with message text:
      - "@{bot_display name} {name}" if ``requires_mention_bot`` is True
      - "{name}" otherwise
    """

    def __init__(self, name, requires_mention_bot, callback, description=""):
        """
        :param name: the command name
        :param requires_mention_bot: if the command requires the bot mention to trigger the slash command
        :param callback: the coroutine to be executed if message text matches
        """
        self._name = name
        self._requires_mention_bot = requires_mention_bot
        self._callback = callback
        self._description = description

    @property
    def name(self) -> str:
        return self._name

    def build_command_description(self) -> str:
        return self._description + " (mention required)" if self._requires_mention_bot \
            else self._description + " (mention not required)"

    def matches(self, context: CommandContext) -> bool:
        pattern = rf"@{context.bot_display_name} {self._name}" if self._requires_mention_bot else rf"{self._name}"
        return pattern == context.text_content

    async def on_activity(self, context: CommandContext):
        await self._callback(context)

    def __eq__(self, o: object) -> bool:
        if isinstance(o, SlashCommandActivity):
            return self._name == o._name and self._requires_mention_bot == o._requires_mention_bot
        return False



import logging

from symphony.bdk.gen.agent_model.v4_user_joined_room import V4UserJoinedRoom

from symphony.bdk.core.activity.api import AbstractActivity
from symphony.bdk.core.activity.command import CommandActivity, CommandContext, SlashCommandActivity
from symphony.bdk.core.activity.form import FormReplyContext, FormReplyActivity
from symphony.bdk.core.activity.user_joined_room import UserJoinedRoomContext, UserJoinedRoomActivity
from symphony.bdk.core.service.datafeed.real_time_event_listener import RealTimeEventListener
from symphony.bdk.core.service.session.session_service import SessionService
from symphony.bdk.gen.agent_model.v4_initiator import V4Initiator
from symphony.bdk.gen.agent_model.v4_message_sent import V4MessageSent
from symphony.bdk.gen.agent_model.v4_symphony_elements_action import V4SymphonyElementsAction

logger = logging.getLogger(__name__)


def _initialize_display_name(func):
    """Decorator around :py:class:`~ActivityRegistry` methods which calls
    :py:meth:`~ActivityRegistry.fetch_bot_display_name` before actually executing the instance method.

    :param func: the function to be decorated
    :return: the decorated function
    """
    async def decorator(*args, **kwargs):
        registry = args[0]
        await registry.fetch_bot_display_name()
        return await func(*args, **kwargs)

    return decorator


class ActivityRegistry(RealTimeEventListener):
    """
    This class allows to bind an py:class::`AbstractActivity` to the Real Time Events source, or Datafeed.
    It also maintains the list of registered activities.
    """

    def __init__(self, session_service: SessionService):
        self._activity_list = []
        self._session_service = session_service
        self._bot_display_name = None

    @property
    def activity_list(self):
        return self._activity_list

    def register(self, activity: AbstractActivity):
        """Registers an activity.

        :param activity: any object inheriting from base :class:`AbstractActivity`
        """
        logger.debug("Registering new activity %s", activity)
        self._pre_process_activity(activity)
        self._activity_list.append(activity)

    def _pre_process_activity(self, activity: AbstractActivity):
        for act in self._activity_list:
            if act == activity:
                self._activity_list.remove(act)
                logger.debug("Activity '%s' has been removed/unsubscribed in order to be replaced", act)

    def slash(self, command: str, mention_bot: bool = True, description: str = ""):
        """Decorator around a listener callback coroutine which takes a
        :py:class:`~symphony.bdk.core.activity.command.CommandContext` as single parameter and returns nothing.
        This registers a new :py:class:`~symphony.bdk.core.activity.command.SlashCommandActivity`
        which executes the decorated coroutine if a message is matching.

        :param command: the command name e.g. "/hello"
        :param mention_bot: if user should mention the bot to trigger the slash command
        :param description: command description
        :return: None
        """
        def decorator(func):
            logger.debug("Registering slash command with command=%s, mention_bot=%s", command, mention_bot)
            self.register(SlashCommandActivity(command, mention_bot, func, description))
            return func

        return decorator

    @_initialize_display_name
    async def on_message_sent(self, initiator: V4Initiator, event: V4MessageSent):
        context = CommandContext(initiator, event, self._bot_display_name)
        for act in self._activity_list:
            if isinstance(act, CommandActivity):
                act.before_matcher(context)
                if act.matches(context):
                    await act.on_activity(context)

    @_initialize_display_name
    async def on_symphony_elements_action(self, initiator: V4Initiator, event: V4SymphonyElementsAction):
        context = FormReplyContext(initiator, event)
        for act in self._activity_list:
            if isinstance(act, FormReplyActivity):
                act.before_matcher(context)
                if act.matches(context):
                    await act.on_activity(context)

    @_initialize_display_name
    async def on_user_joined_room(self, initiator: V4Initiator, event: V4UserJoinedRoom):
        context = UserJoinedRoomContext(initiator, event)
        for act in self._activity_list:
            if isinstance(act, UserJoinedRoomActivity):
                act.before_matcher(context)
                if act.matches(context):
                    await act.on_activity(context)

    async def fetch_bot_display_name(self):
        """Fetches the bot display name if not already done.

        :return: None
        """
        if self._bot_display_name is None:
            session = await self._session_service.get_session()
            self._bot_display_name = session.display_name
            logger.debug("Bot display name is : %s", self._bot_display_name)



"""
    Login API

    For bots and other on-premise processes to authenticate. Once authenticated, the bot will be able to use the methods described in serviceAPI.yaml and agentAPI.yaml.  Authentication requests will expect the user to pass a token containing user identification information and signed by the user's private key  There will be two implementations of this API, one on your Pod and one on the Key Manager. In order to fully authenticate, an API client will have to call both of these implementations and pass both of the session tokens returned as headers in all subsequent requests to the Symphony API.   # noqa: E501

    The version of the OpenAPI document: 20.12.1
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from symphony.bdk.gen.api_client import ApiClient, Endpoint as _Endpoint
from symphony.bdk.gen.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from symphony.bdk.gen.login_model.authenticate_extension_app_request import AuthenticateExtensionAppRequest
from symphony.bdk.gen.login_model.authenticate_request import AuthenticateRequest
from symphony.bdk.gen.login_model.error import Error
from symphony.bdk.gen.login_model.extension_app_tokens import ExtensionAppTokens
from symphony.bdk.gen.login_model.token import Token


class AuthenticationApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __pubkey_app_authenticate_post(
            self,
            authenticate_request,
            **kwargs
        ):
            """Authenticate an App with public key  # noqa: E501

            Based on an authentication request token signed by the application's RSA private key, authenticate the API caller and return a session token.  A HTTP 401 Unauthorized error is returned on errors during authentication (e.g. invalid app, malformed authentication token, app's public key not imported in the pod, invalid token signature etc.).   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = login_api.pubkey_app_authenticate_post(authenticate_request, async_req=True)
            >>> result = thread.get()

            Args:
                authenticate_request (AuthenticateRequest):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Token
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['authenticate_request'] = \
                authenticate_request
            return self.call_with_http_info(**kwargs)

        self.pubkey_app_authenticate_post = _Endpoint(
            settings={
                'response_type': (Token,),
                'auth': [],
                'endpoint_path': '/pubkey/app/authenticate',
                'operation_id': 'pubkey_app_authenticate_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'authenticate_request',
                ],
                'required': [
                    'authenticate_request',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'authenticate_request':
                        (AuthenticateRequest,),
                },
                'attribute_map': {
                },
                'location_map': {
                    'authenticate_request': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__pubkey_app_authenticate_post
        )

        def __pubkey_app_user_user_id_authenticate_post(
            self,
            session_token,
            user_id,
            **kwargs
        ):
            """Authenticate an application in a delegated context to act on behalf of a user  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = login_api.pubkey_app_user_user_id_authenticate_post(session_token, user_id, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): App Session authentication token.
                user_id (int): the user ID

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Token
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['user_id'] = \
                user_id
            return self.call_with_http_info(**kwargs)

        self.pubkey_app_user_user_id_authenticate_post = _Endpoint(
            settings={
                'response_type': (Token,),
                'auth': [],
                'endpoint_path': '/pubkey/app/user/{userId}/authenticate',
                'operation_id': 'pubkey_app_user_user_id_authenticate_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'user_id',
                ],
                'required': [
                    'session_token',
                    'user_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'user_id':
                        (int,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'user_id': 'userId',
                },
                'location_map': {
                    'session_token': 'header',
                    'user_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__pubkey_app_user_user_id_authenticate_post
        )

        def __pubkey_app_username_username_authenticate_post(
            self,
            session_token,
            username,
            **kwargs
        ):
            """Authenticate an application in a delegated context to act on behalf of a user  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = login_api.pubkey_app_username_username_authenticate_post(session_token, username, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): App Session authentication token.
                username (str): the username

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Token
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['username'] = \
                username
            return self.call_with_http_info(**kwargs)

        self.pubkey_app_username_username_authenticate_post = _Endpoint(
            settings={
                'response_type': (Token,),
                'auth': [],
                'endpoint_path': '/pubkey/app/username/{username}/authenticate',
                'operation_id': 'pubkey_app_username_username_authenticate_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'username',
                ],
                'required': [
                    'session_token',
                    'username',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'username':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'username': 'username',
                },
                'location_map': {
                    'session_token': 'header',
                    'username': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__pubkey_app_username_username_authenticate_post
        )

        def __pubkey_authenticate_post(
            self,
            authenticate_request,
            **kwargs
        ):
            """Authenticate with public key  # noqa: E501

            Based on an authentication request token signed by the caller's RSA private key, authenticate the API caller and return a session token.  A HTTP 401 Unauthorized error is returned on errors during authentication (e.g. invalid user, malformed authentication token, user's public key not imported in the pod, invalid token signature etc.).   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = login_api.pubkey_authenticate_post(authenticate_request, async_req=True)
            >>> result = thread.get()

            Args:
                authenticate_request (AuthenticateRequest):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Token
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['authenticate_request'] = \
                authenticate_request
            return self.call_with_http_info(**kwargs)

        self.pubkey_authenticate_post = _Endpoint(
            settings={
                'response_type': (Token,),
                'auth': [],
                'endpoint_path': '/pubkey/authenticate',
                'operation_id': 'pubkey_authenticate_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'authenticate_request',
                ],
                'required': [
                    'authenticate_request',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'authenticate_request':
                        (AuthenticateRequest,),
                },
                'attribute_map': {
                },
                'location_map': {
                    'authenticate_request': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__pubkey_authenticate_post
        )

        def __v1_pubkey_app_authenticate_extension_app_post(
            self,
            authenticate_request,
            **kwargs
        ):
            """Authenticate extension app with public key  # noqa: E501

            Based on an authentication request token signed by the caller's RSA private key, authenticate the API caller and return a session token.  A HTTP 401 Unauthorized error is returned on errors during authentication (e.g. invalid user, malformed authentication token, user's public key not imported in the pod, invalid token signature etc.).   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = login_api.v1_pubkey_app_authenticate_extension_app_post(authenticate_request, async_req=True)
            >>> result = thread.get()

            Args:
                authenticate_request (AuthenticateExtensionAppRequest):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                ExtensionAppTokens
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['authenticate_request'] = \
                authenticate_request
            return self.call_with_http_info(**kwargs)

        self.v1_pubkey_app_authenticate_extension_app_post = _Endpoint(
            settings={
                'response_type': (ExtensionAppTokens,),
                'auth': [],
                'endpoint_path': '/v1/pubkey/app/authenticate/extensionApp',
                'operation_id': 'v1_pubkey_app_authenticate_extension_app_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'authenticate_request',
                ],
                'required': [
                    'authenticate_request',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'authenticate_request':
                        (AuthenticateExtensionAppRequest,),
                },
                'attribute_map': {
                },
                'location_map': {
                    'authenticate_request': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v1_pubkey_app_authenticate_extension_app_post
        )



"""
    Agent API

    This document refers to Symphony API calls to send and receive messages and content. They need the on-premise Agent installed to perform decryption/encryption of content.  - sessionToken and keyManagerToken can be obtained by calling the authenticationAPI on the symphony back end and the key manager respectively. Refer to the methods described in authenticatorAPI.yaml. - Actions are defined to be atomic, ie will succeed in their entirety or fail and have changed nothing. - If it returns a 40X status then it will have sent no message to any stream even if a request to aome subset of the requested streams would have succeeded. - If this contract cannot be met for any reason then this is an error and the response code will be 50X. - MessageML is a markup language for messages. See reference here: https://rest-api.symphony.com/docs/messagemlv2 - **Real Time Events**: The following events are returned when reading from a real time messages and events stream (\"datafeed\"). These events will be returned for datafeeds created with the v5 endpoints. To know more about the endpoints, refer to Create Messages/Events Stream and Read Messages/Events Stream. Unless otherwise specified, all events were added in 1.46.   # noqa: E501

    The version of the OpenAPI document: 20.12.0
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from symphony.bdk.gen.api_client import ApiClient, Endpoint as _Endpoint
from symphony.bdk.gen.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from symphony.bdk.gen.agent_model.message_search_query import MessageSearchQuery
from symphony.bdk.gen.agent_model.v2_error import V2Error
from symphony.bdk.gen.agent_model.v4_import_response_list import V4ImportResponseList
from symphony.bdk.gen.agent_model.v4_message import V4Message
from symphony.bdk.gen.agent_model.v4_message_blast_response import V4MessageBlastResponse
from symphony.bdk.gen.agent_model.v4_message_import_list import V4MessageImportList
from symphony.bdk.gen.agent_model.v4_message_list import V4MessageList


class MessagesApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __v1_message_id_get(
            self,
            session_token,
            key_manager_token,
            id,
            **kwargs
        ):
            """Get a message by ID  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = agent_api.v1_message_id_get(session_token, key_manager_token, id, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                key_manager_token (str): Key Manager authentication token.
                id (str): Message ID as a URL-safe string

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V4Message
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['key_manager_token'] = \
                key_manager_token
            kwargs['id'] = \
                id
            return self.call_with_http_info(**kwargs)

        self.v1_message_id_get = _Endpoint(
            settings={
                'response_type': (V4Message,),
                'auth': [],
                'endpoint_path': '/v1/message/{id}',
                'operation_id': 'v1_message_id_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'key_manager_token',
                    'id',
                ],
                'required': [
                    'session_token',
                    'key_manager_token',
                    'id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'key_manager_token':
                        (str,),
                    'id':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'key_manager_token': 'keyManagerToken',
                    'id': 'id',
                },
                'location_map': {
                    'session_token': 'header',
                    'key_manager_token': 'header',
                    'id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_message_id_get
        )

        def __v1_message_search_get(
            self,
            query,
            session_token,
            key_manager_token,
            **kwargs
        ):
            """Search messages  # noqa: E501

            Search messages according to the specified criteria. The \"query\" parameter takes a search query defined as \"field:value\" pairs combined by the operator \"AND\" (e.g. \"text:foo AND autor:bar\"). Supported fields are  (case-insensitive): \"text\", \"author\", \"hashtag\", \"cashtag\", \"mention\", \"signal\", \"fromDate\", \"toDate\",  \"streamId\", \"streamType\".  \"text\" search requires a \"streamId\" to be specified.  \"streamType\" accepts one of the following values: \"chat\" (IMs and MIMs), \"im\", \"mim\", \"chatroom\", \"post\".  \"signal\" queries can only be combined with \"fromDate\", \"toDate\", \"skip\" and \"limit\" parameters.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = agent_api.v1_message_search_get(query, session_token, key_manager_token, async_req=True)
            >>> result = thread.get()

            Args:
                query (str): The search query. See above for the query syntax.
                session_token (str): Session authentication token.
                key_manager_token (str): Key Manager authentication token.

            Keyword Args:
                skip (int): No. of results to skip. . [optional]
                limit (int): Max no. of results to return. If no value is provided, 50 is the default. . [optional]
                scope (str): Describes where content should be searched for that query. It can exclusively apply to Symphony content or to one Connector. . [optional]
                sort_dir (str): Messages sort direction : ASC or DESC (default to DESC) . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V4MessageList
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['query'] = \
                query
            kwargs['session_token'] = \
                session_token
            kwargs['key_manager_token'] = \
                key_manager_token
            return self.call_with_http_info(**kwargs)

        self.v1_message_search_get = _Endpoint(
            settings={
                'response_type': (V4MessageList,),
                'auth': [],
                'endpoint_path': '/v1/message/search',
                'operation_id': 'v1_message_search_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'query',
                    'session_token',
                    'key_manager_token',
                    'skip',
                    'limit',
                    'scope',
                    'sort_dir',
                ],
                'required': [
                    'query',
                    'session_token',
                    'key_manager_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'query':
                        (str,),
                    'session_token':
                        (str,),
                    'key_manager_token':
                        (str,),
                    'skip':
                        (int,),
                    'limit':
                        (int,),
                    'scope':
                        (str,),
                    'sort_dir':
                        (str,),
                },
                'attribute_map': {
                    'query': 'query',
                    'session_token': 'sessionToken',
                    'key_manager_token': 'keyManagerToken',
                    'skip': 'skip',
                    'limit': 'limit',
                    'scope': 'scope',
                    'sort_dir': 'sortDir',
                },
                'location_map': {
                    'query': 'query',
                    'session_token': 'header',
                    'key_manager_token': 'header',
                    'skip': 'query',
                    'limit': 'query',
                    'scope': 'query',
                    'sort_dir': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_message_search_get
        )

        def __v1_message_search_post(
            self,
            session_token,
            key_manager_token,
            query,
            **kwargs
        ):
            """Search messages  # noqa: E501

            Search messages according to the specified criteria.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = agent_api.v1_message_search_post(session_token, key_manager_token, query, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                key_manager_token (str): Key Manager authentication token.
                query (MessageSearchQuery): The search query. See above for the query syntax.

            Keyword Args:
                skip (int): No. of results to skip. . [optional]
                limit (int): Max no. of results to return. If no value is provided, 50 is the default. . [optional]
                scope (str): Describes where content should be searched for that query. It can exclusively apply to Symphony content or to one Connector. . [optional]
                sort_dir (str): Messages sort direction : ASC or DESC (default to DESC) . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V4MessageList
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['key_manager_token'] = \
                key_manager_token
            kwargs['query'] = \
                query
            return self.call_with_http_info(**kwargs)

        self.v1_message_search_post = _Endpoint(
            settings={
                'response_type': (V4MessageList,),
                'auth': [],
                'endpoint_path': '/v1/message/search',
                'operation_id': 'v1_message_search_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'key_manager_token',
                    'query',
                    'skip',
                    'limit',
                    'scope',
                    'sort_dir',
                ],
                'required': [
                    'session_token',
                    'key_manager_token',
                    'query',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'key_manager_token':
                        (str,),
                    'query':
                        (MessageSearchQuery,),
                    'skip':
                        (int,),
                    'limit':
                        (int,),
                    'scope':
                        (str,),
                    'sort_dir':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'key_manager_token': 'keyManagerToken',
                    'skip': 'skip',
                    'limit': 'limit',
                    'scope': 'scope',
                    'sort_dir': 'sortDir',
                },
                'location_map': {
                    'session_token': 'header',
                    'key_manager_token': 'header',
                    'query': 'body',
                    'skip': 'query',
                    'limit': 'query',
                    'scope': 'query',
                    'sort_dir': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v1_message_search_post
        )

        def __v4_message_blast_post(
            self,
            session_token,
            sids,
            **kwargs
        ):
            """Post a message to multiple existing streams.  # noqa: E501

            Post a new message to the given list of streams. The stream can be a chatroom, an IM or a multiparty IM.  You may include an attachment on the message.  The message can be provided as MessageMLV2 or PresentationML. Both formats support Freemarker templates.  The optional parameter \"data\" can be used to provide a JSON payload containing entity data. If the message contains explicit references to entity data (in \"data-entity-id\" element attributes), this parameter is required.  If the message is in MessageML and fails schema validation a client error results  This endpoint is idempotent, it means that a 200 response will be returned even if the message has not been delivered to some streams. Check the `errors` map from the response in order to see on which stream(s) the message has not been delivered.  The maximum number of streams where the message can be sent is limitted to 100.  Regarding authentication, you must either use the sessionToken which was created for delegated app access or both the sessionToken and keyManagerToken together.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = agent_api.v4_message_blast_post(session_token, sids, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Authorization token used to make delegated calls.
                sids ([str]): A comma-separated list of Stream IDs

            Keyword Args:
                key_manager_token (str): Key Manager authentication token.. [optional]
                message (str): The message payload in MessageML.. [optional]
                data (str): Optional message data in EntityJSON.. [optional]
                version (str): Optional message version in the format \\\"major.minor\\\". If empty, defaults to the latest supported version. . [optional]
                attachment (file_type): Optional file attachment.. [optional]
                preview (file_type): Optional attachment preview.. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V4MessageBlastResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['sids'] = \
                sids
            return self.call_with_http_info(**kwargs)

        self.v4_message_blast_post = _Endpoint(
            settings={
                'response_type': (V4MessageBlastResponse,),
                'auth': [],
                'endpoint_path': '/v4/message/blast',
                'operation_id': 'v4_message_blast_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'sids',
                    'key_manager_token',
                    'message',
                    'data',
                    'version',
                    'attachment',
                    'preview',
                ],
                'required': [
                    'session_token',
                    'sids',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'sids':
                        ([str],),
                    'key_manager_token':
                        (str,),
                    'message':
                        (str,),
                    'data':
                        (str,),
                    'version':
                        (str,),
                    'attachment':
                        (file_type,),
                    'preview':
                        (file_type,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'sids': 'sids',
                    'key_manager_token': 'keyManagerToken',
                    'message': 'message',
                    'data': 'data',
                    'version': 'version',
                    'attachment': 'attachment',
                    'preview': 'preview',
                },
                'location_map': {
                    'session_token': 'header',
                    'sids': 'form',
                    'key_manager_token': 'header',
                    'message': 'form',
                    'data': 'form',
                    'version': 'form',
                    'attachment': 'form',
                    'preview': 'form',
                },
                'collection_format_map': {
                    'sids': 'csv',
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'multipart/form-data'
                ]
            },
            api_client=api_client,
            callable=__v4_message_blast_post
        )

        def __v4_message_import_post(
            self,
            session_token,
            key_manager_token,
            message_list,
            **kwargs
        ):
            """Import messages from other systems into Symphony.  # noqa: E501

            Sends a message to be imported into the system. Allows you to override the timestamp and author of the message with your desired values. The requesting user must have the Content Management role. The user that the message is intended to have come from must also be present in the conversation. The intended message timestamp must be a valid time from the past. It cannot be a future timestamp. Optionally the original message ID can be specified to identify the imported message for the purpose of repeat imports.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = agent_api.v4_message_import_post(session_token, key_manager_token, message_list, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                key_manager_token (str): Key Manager authentication token.
                message_list (V4MessageImportList):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V4ImportResponseList
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['key_manager_token'] = \
                key_manager_token
            kwargs['message_list'] = \
                message_list
            return self.call_with_http_info(**kwargs)

        self.v4_message_import_post = _Endpoint(
            settings={
                'response_type': (V4ImportResponseList,),
                'auth': [],
                'endpoint_path': '/v4/message/import',
                'operation_id': 'v4_message_import_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'key_manager_token',
                    'message_list',
                ],
                'required': [
                    'session_token',
                    'key_manager_token',
                    'message_list',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'key_manager_token':
                        (str,),
                    'message_list':
                        (V4MessageImportList,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'key_manager_token': 'keyManagerToken',
                },
                'location_map': {
                    'session_token': 'header',
                    'key_manager_token': 'header',
                    'message_list': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v4_message_import_post
        )

        def __v4_stream_sid_message_create_post(
            self,
            sid,
            session_token,
            **kwargs
        ):
            """Post a message to one existing stream.  # noqa: E501

            Post a new message to the given stream. The stream can be a chatroom, an IM or a multiparty IM.  You may include an attachment on the message.  The message can be provided as MessageMLV2 or PresentationML. Both formats support Freemarker templates.  The optional parameter \"data\" can be used to provide a JSON payload containing entity data. If the message contains explicit references to entity data (in \"data-entity-id\" element attributes), this parameter is required.  If the message is in MessageML and fails schema validation a client error results  If the message is sent then 200 is returned.  Regarding authentication, you must either use the sessionToken which was created for delegated app access or both the sessionToken and keyManagerToken together.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = agent_api.v4_stream_sid_message_create_post(sid, session_token, async_req=True)
            >>> result = thread.get()

            Args:
                sid (str): Stream ID
                session_token (str): Authorization token used to make delegated calls.

            Keyword Args:
                key_manager_token (str): Key Manager authentication token.. [optional]
                message (str): The message payload in MessageML.. [optional]
                data (str): Optional message data in EntityJSON.. [optional]
                version (str): Optional message version in the format \\\"major.minor\\\". If empty, defaults to the latest supported version. . [optional]
                attachment (file_type): Optional file attachment.. [optional]
                preview (file_type): Optional attachment preview.. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V4Message
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['sid'] = \
                sid
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v4_stream_sid_message_create_post = _Endpoint(
            settings={
                'response_type': (V4Message,),
                'auth': [],
                'endpoint_path': '/v4/stream/{sid}/message/create',
                'operation_id': 'v4_stream_sid_message_create_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'sid',
                    'session_token',
                    'key_manager_token',
                    'message',
                    'data',
                    'version',
                    'attachment',
                    'preview',
                ],
                'required': [
                    'sid',
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'sid':
                        (str,),
                    'session_token':
                        (str,),
                    'key_manager_token':
                        (str,),
                    'message':
                        (str,),
                    'data':
                        (str,),
                    'version':
                        (str,),
                    'attachment':
                        (file_type,),
                    'preview':
                        (file_type,),
                },
                'attribute_map': {
                    'sid': 'sid',
                    'session_token': 'sessionToken',
                    'key_manager_token': 'keyManagerToken',
                    'message': 'message',
                    'data': 'data',
                    'version': 'version',
                    'attachment': 'attachment',
                    'preview': 'preview',
                },
                'location_map': {
                    'sid': 'path',
                    'session_token': 'header',
                    'key_manager_token': 'header',
                    'message': 'form',
                    'data': 'form',
                    'version': 'form',
                    'attachment': 'form',
                    'preview': 'form',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'multipart/form-data'
                ]
            },
            api_client=api_client,
            callable=__v4_stream_sid_message_create_post
        )

        def __v4_stream_sid_message_get(
            self,
            sid,
            since,
            session_token,
            key_manager_token,
            **kwargs
        ):
            """Get messages from an existing stream.  # noqa: E501

            A caller can fetch all unseen messages by passing the timestamp of the last message seen as the since parameter and the number of messages with the same timestamp value already seen as the skip parameter. This means that every message will be seen exactly once even in the case that an additional message is processed with the same timestamp as the last message returned by the previous call, and the case where there are more than maxMessages with the same timestamp value.  This method is intended for historic queries and is generally reliable but if guaranteed delivery of every message in real time is required then the equivilent firehose method should be called.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = agent_api.v4_stream_sid_message_get(sid, since, session_token, key_manager_token, async_req=True)
            >>> result = thread.get()

            Args:
                sid (str): Stream ID 
                since (int): Timestamp of first required message.  This is a long integer value representing milliseconds since Jan 1 1970 
                session_token (str): Session authentication token.
                key_manager_token (str): Key Manager authentication token.

            Keyword Args:
                skip (int): No. of messages to skip. . [optional]
                limit (int): Max No. of messages to return. If no value is provided, 50 is the default. The maximum supported value is 500. . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V4MessageList
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['sid'] = \
                sid
            kwargs['since'] = \
                since
            kwargs['session_token'] = \
                session_token
            kwargs['key_manager_token'] = \
                key_manager_token
            return self.call_with_http_info(**kwargs)

        self.v4_stream_sid_message_get = _Endpoint(
            settings={
                'response_type': (V4MessageList,),
                'auth': [],
                'endpoint_path': '/v4/stream/{sid}/message',
                'operation_id': 'v4_stream_sid_message_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'sid',
                    'since',
                    'session_token',
                    'key_manager_token',
                    'skip',
                    'limit',
                ],
                'required': [
                    'sid',
                    'since',
                    'session_token',
                    'key_manager_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'sid':
                        (str,),
                    'since':
                        (int,),
                    'session_token':
                        (str,),
                    'key_manager_token':
                        (str,),
                    'skip':
                        (int,),
                    'limit':
                        (int,),
                },
                'attribute_map': {
                    'sid': 'sid',
                    'since': 'since',
                    'session_token': 'sessionToken',
                    'key_manager_token': 'keyManagerToken',
                    'skip': 'skip',
                    'limit': 'limit',
                },
                'location_map': {
                    'sid': 'path',
                    'since': 'query',
                    'session_token': 'header',
                    'key_manager_token': 'header',
                    'skip': 'query',
                    'limit': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v4_stream_sid_message_get
        )



import enum


class RoleId(enum.Enum):
    """Static roles that have special logic attached to them.
    See: `Symphony Roles <https://developers.symphony.com/restapi/docs/permissions>`_
    """
    ADMINISTRATOR = "ADMINISTRATOR"
    SUPER_ADMINISTRATOR = "SUPER_ADMINISTRATOR"
    COMPLIANCE_OFFICER = "COMPLIANCE_OFFICER"
    SUPER_COMPLIANCE_OFFICER = "SUPER_COMPLIANCE_OFFICER"
    L1_SUPPORT = "L1_SUPPORT"
    L2_SUPPORT = "L2_SUPPORT"
    USER_PROVISIONING = "USER_PROVISIONING"
    CONTENT_EXPORT_SERVICE = "CONTENT_EXPORT_SERVICE"
    INDIVIDUAL = "INDIVIDUAL"
    SYMPHONY_ADMIN = "SYMPHONY_ADMIN"
    KEY_MANAGER = "KEY_MANAGER"
    CONTENT_MANAGEMENT = "CONTENT_MANAGEMENT"
    EF_POLICY_MANAGEMENT = "EF_POLICY_MANAGEMENT"
    MALWARE_SCAN_MANAGER = "MALWARE_SCAN_MANAGER"
    MALWARE_SCAN_STATE_USER = "MALWARE_SCAN_STATE_USER"
    AGENT = "AGENT"
    SCOPE_MANAGEMENT = "SCOPE_MANAGEMENT"
    AUDIT_TRAIL_MANAGEMENT = "AUDIT_TRAIL_MANAGEMENT"
    CEP_VISIBILITY_GROUP_MANAGEMENT = "CEP_VISIBILITY_GROUP_MANAGEMENT"



"""Module containing all authentication related exception.
"""


class AuthInitializationError(Exception):
    """Thrown when unable to read/parse a RSA Private Key or a certificate.
    """

    def __init__(self, message: str):
        super().__init__()
        self.message = message


class AuthUnauthorizedError(Exception):
    """When thrown, it means that authentication cannot be performed for several reasons
    """

    def __init__(self, message: str, cause: Exception = None):
        super().__init__()
        self.message = message
        self.cause = cause



import asyncio

try:
    from inspect import iscoroutinefunction
except ImportError:
    iscoroutinefunction = None

import sys


import six

from tenacity import AttemptManager, RetryAction, TryAgain
from tenacity import BaseRetrying
from tenacity import DoAttempt
from tenacity import DoSleep
from tenacity import RetryCallState


class AsyncRetrying(BaseRetrying):
    """This is a modified version of the tenacity.AsyncRetrying class that handles asynchronously defined retry or
    retry_error_callback parameters.

    There is an open PR for this change on the tenacity repository: https://github.com/jd/tenacity/pull/289
    """
    def __init__(self, sleep=asyncio.sleep, **kwargs):
        super().__init__(**kwargs)
        self.sleep = sleep

    async def iter(self, retry_state):
        fut = retry_state.outcome
        if fut is None:
            if self.before is not None:
                self.before(retry_state)
            return DoAttempt()

        is_explicit_retry = retry_state.outcome.failed and isinstance(
            retry_state.outcome.exception(), TryAgain
        )
        if iscoroutinefunction(self.retry):
            should_retry = await self.retry(retry_state=retry_state)
        else:
            should_retry = self.retry(retry_state=retry_state)
        if not (is_explicit_retry or should_retry):
            return fut.result()

        if self.after is not None:
            self.after(retry_state=retry_state)

        self.statistics["delay_since_first_attempt"] = retry_state.seconds_since_start
        if self.stop(retry_state=retry_state):
            if self.retry_error_callback:
                if iscoroutinefunction(self.retry_error_callback):
                    return await self.retry_error_callback(retry_state=retry_state)
                return self.retry_error_callback(retry_state=retry_state)
            retry_exc = self.retry_error_cls(fut)
            if self.reraise:
                raise retry_exc.reraise()
            six.raise_from(retry_exc, fut.exception())

        if self.wait:
            iteration_sleep = self.wait(retry_state=retry_state)
        else:
            iteration_sleep = 0.0
        retry_state.next_action = RetryAction(iteration_sleep)
        retry_state.idle_for += iteration_sleep
        self.statistics["idle_for"] += iteration_sleep
        self.statistics["attempt_number"] += 1

        if self.before_sleep is not None:
            self.before_sleep(retry_state=retry_state)

        return DoSleep(iteration_sleep)

    async def __call__(self, fn, *args, **kwargs):
        self.begin(fn)

        retry_state = RetryCallState(retry_object=self, fn=fn, args=args, kwargs=kwargs)
        while True:
            do = await self.iter(retry_state=retry_state)
            if isinstance(do, DoAttempt):
                try:
                    result = await fn(*args, **kwargs)
                except BaseException:
                    retry_state.set_exception(sys.exc_info())
                else:
                    retry_state.set_result(result)
            elif isinstance(do, DoSleep):
                retry_state.prepare_for_next_attempt()
                await self.sleep(do)
            else:
                return do

    def __aiter__(self):
        self.begin(None)
        self._retry_state = RetryCallState(self, fn=None, args=(), kwargs={})
        return self

    async def __anext__(self):
        while True:
            do = await self.iter(retry_state=self._retry_state)
            if do is None:
                raise StopAsyncIteration
            if isinstance(do, DoAttempt):
                return AttemptManager(retry_state=self._retry_state)
            if isinstance(do, DoSleep):
                self._retry_state.prepare_for_next_attempt()
                await self.sleep(do)
            else:
                return do

    def wraps(self, fn):
        fn = super().wraps(fn)

        # Ensure wrapper is recognized as a coroutine function.

        async def async_wrapped(*args, **kwargs):
            return await fn(*args, **kwargs)

        # Preserve attributes
        async_wrapped.retry = fn.retry
        async_wrapped.retry_with = fn.retry_with

        return async_wrapped



"""
    Pod API

    This document refers to Symphony API calls that do not need encryption or decryption of content.  - sessionToken can be obtained by calling the authenticationAPI on the symphony back end and the key manager respectively. Refer to the methods described in authenticatorAPI.yaml. - Actions are defined to be atomic, ie will succeed in their entirety or fail and have changed nothing. - If it returns a 40X status then it will have made no change to the system even if ome subset of the request would have succeeded. - If this contract cannot be met for any reason then this is an error and the response code will be 50X.   # noqa: E501

    The version of the OpenAPI document: 20.12.1
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from symphony.bdk.gen.api_client import ApiClient, Endpoint as _Endpoint
from symphony.bdk.gen.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from symphony.bdk.gen.pod_model.error import Error
from symphony.bdk.gen.pod_model.file_extension import FileExtension
from symphony.bdk.gen.pod_model.file_extensions_response import FileExtensionsResponse
from symphony.bdk.gen.pod_model.message_details import MessageDetails
from symphony.bdk.gen.pod_model.message_ids import MessageIds
from symphony.bdk.gen.pod_model.message_metadata_response import MessageMetadataResponse
from symphony.bdk.gen.pod_model.message_receipt_detail_response import MessageReceiptDetailResponse


class DefaultApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __delete_allowed_file_extension(
            self,
            session_token,
            extension,
            **kwargs
        ):
            """Allows deletion of a specific file extension supported for upload  # noqa: E501

            Provides a method to delete a specific file extension configured for upload support via an admin. The file extension identifying the resource is treated case-insensitively by the API.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.delete_allowed_file_extension(session_token, extension, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                extension (str):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                None
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['extension'] = \
                extension
            return self.call_with_http_info(**kwargs)

        self.delete_allowed_file_extension = _Endpoint(
            settings={
                'response_type': None,
                'auth': [],
                'endpoint_path': '/file_ext/v1/allowed_extensions/{extension}',
                'operation_id': 'delete_allowed_file_extension',
                'http_method': 'DELETE',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'extension',
                ],
                'required': [
                    'session_token',
                    'extension',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'extension':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'extension': 'extension',
                },
                'location_map': {
                    'session_token': 'header',
                    'extension': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__delete_allowed_file_extension
        )

        def __list_allowed_file_extensions(
            self,
            **kwargs
        ):
            """Allows iteration of all file extensions supported for upload  # noqa: E501

            Provides a RESTful API to iterate all file extensions configured by the tenant admin that are allowed for the upload.  Pagination of this list is managed through a combination of the optional request parameters and service-side managed maximums.  Pagination of the results is provided through the before or after input paramters and presented through the opaque cursor values provided as output from a previous response.  Only one of before or after or neither may be provided.  DO NOT store cursors. Cursors can quickly become invalid if items are added or deleted. Use them only during a short-period of time that you are traversing the list.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.list_allowed_file_extensions(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                limit (int): This is the maximum number of objects that may be returned. A query may return fewer than the value of limit due to filtering or service-side maximums. Do not depend on the number of results being fewer than the limit value to indicate your query reached the end of the list of data, use the absence of next instead as described below. For example, if you set limit to 10 and 9 results are returned, there may be more data available, but one item was removed due to privacy filtering. Some maximums for limit may be enforced for performance reasons. In all cases, the API returns the correct pagination links. . [optional]
                before (str): Returns results from an opaque \"before\" cursor value as presented via a response cursor . [optional]
                after (str): Returns results from an opaque \"after\" cursor value as presented via a response cursor . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                FileExtensionsResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        self.list_allowed_file_extensions = _Endpoint(
            settings={
                'response_type': (FileExtensionsResponse,),
                'auth': [],
                'endpoint_path': '/file_ext/v1/allowed_extensions',
                'operation_id': 'list_allowed_file_extensions',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'limit',
                    'before',
                    'after',
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'limit':
                        (int,),
                    'before':
                        (str,),
                    'after':
                        (str,),
                },
                'attribute_map': {
                    'limit': 'limit',
                    'before': 'before',
                    'after': 'after',
                },
                'location_map': {
                    'limit': 'query',
                    'before': 'query',
                    'after': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__list_allowed_file_extensions
        )

        def __put_allowed_file_extension(
            self,
            session_token,
            extension,
            v3_file_extension,
            **kwargs
        ):
            """Allows replacement or creation of a specific file extension supported for upload  # noqa: E501

            Provides a method to create or replace a specific file extension configured for upload support via an admin. The API treats the file extension in the path case-insensitively by converting it to lowecase.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.put_allowed_file_extension(session_token, extension, v3_file_extension, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                extension (str):
                v3_file_extension (FileExtension):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                FileExtension
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['extension'] = \
                extension
            kwargs['v3_file_extension'] = \
                v3_file_extension
            return self.call_with_http_info(**kwargs)

        self.put_allowed_file_extension = _Endpoint(
            settings={
                'response_type': (FileExtension,),
                'auth': [],
                'endpoint_path': '/file_ext/v1/allowed_extensions/{extension}',
                'operation_id': 'put_allowed_file_extension',
                'http_method': 'PUT',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'extension',
                    'v3_file_extension',
                ],
                'required': [
                    'session_token',
                    'extension',
                    'v3_file_extension',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'extension':
                        (str,),
                    'v3_file_extension':
                        (FileExtension,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'extension': 'extension',
                },
                'location_map': {
                    'session_token': 'header',
                    'extension': 'path',
                    'v3_file_extension': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__put_allowed_file_extension
        )

        def __v1_admin_messages_message_id_metadata_relationships_get(
            self,
            session_token,
            user_agent,
            message_id,
            **kwargs
        ):
            """Get the message metadata relationship  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_admin_messages_message_id_metadata_relationships_get(session_token, user_agent, message_id, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                user_agent (str): Request user agent.
                message_id (str): Valid id of a SOCIALMESSAGE

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                MessageMetadataResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['user_agent'] = \
                user_agent
            kwargs['message_id'] = \
                message_id
            return self.call_with_http_info(**kwargs)

        self.v1_admin_messages_message_id_metadata_relationships_get = _Endpoint(
            settings={
                'response_type': (MessageMetadataResponse,),
                'auth': [],
                'endpoint_path': '/v1/admin/messages/{messageId}/metadata/relationships',
                'operation_id': 'v1_admin_messages_message_id_metadata_relationships_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'user_agent',
                    'message_id',
                ],
                'required': [
                    'session_token',
                    'user_agent',
                    'message_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'user_agent':
                        (str,),
                    'message_id':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'user_agent': 'User-Agent',
                    'message_id': 'messageId',
                },
                'location_map': {
                    'session_token': 'header',
                    'user_agent': 'header',
                    'message_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_admin_messages_message_id_metadata_relationships_get
        )

        def __v1_admin_messages_message_id_receipts_get(
            self,
            session_token,
            message_id,
            **kwargs
        ):
            """Fetch receipts details from a specific message.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_admin_messages_message_id_receipts_get(session_token, message_id, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                message_id (str): The message ID

            Keyword Args:
                before (str): Returns results from an opaque “before” cursor value as presented via a response cursor.. [optional]
                after (str): Returns results from an opaque “after” cursor value as presented via a response cursor.. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                MessageReceiptDetailResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['message_id'] = \
                message_id
            return self.call_with_http_info(**kwargs)

        self.v1_admin_messages_message_id_receipts_get = _Endpoint(
            settings={
                'response_type': (MessageReceiptDetailResponse,),
                'auth': [],
                'endpoint_path': '/v1/admin/messages/{messageId}/receipts',
                'operation_id': 'v1_admin_messages_message_id_receipts_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'message_id',
                    'before',
                    'after',
                ],
                'required': [
                    'session_token',
                    'message_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'message_id':
                        (str,),
                    'before':
                        (str,),
                    'after':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'message_id': 'messageId',
                    'before': 'before',
                    'after': 'after',
                },
                'location_map': {
                    'session_token': 'header',
                    'message_id': 'path',
                    'before': 'query',
                    'after': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_admin_messages_message_id_receipts_get
        )

        def __v1_admin_messages_post(
            self,
            session_token,
            message_ids,
            **kwargs
        ):
            """Fetch message details  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_admin_messages_post(session_token, message_ids, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token
                message_ids (MessageIds): the ids of the messages to be fetched

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                MessageDetails
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['message_ids'] = \
                message_ids
            return self.call_with_http_info(**kwargs)

        self.v1_admin_messages_post = _Endpoint(
            settings={
                'response_type': (MessageDetails,),
                'auth': [],
                'endpoint_path': '/v1/admin/messages',
                'operation_id': 'v1_admin_messages_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'message_ids',
                ],
                'required': [
                    'session_token',
                    'message_ids',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'message_ids':
                        (MessageIds,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'session_token': 'header',
                    'message_ids': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v1_admin_messages_post
        )



import json
from pathlib import Path

import yaml

from symphony.bdk.core.config.exception import BdkConfigError
from symphony.bdk.core.config.model.bdk_config import BdkConfig


class BdkConfigLoader:
    """ Config loader class

    Provide methods to load a JSON or YAML configuration
    from an absolute path or `$HOME/.symphony``
    directory or string object to a BdkConfig object.
    """

    @classmethod
    def load_from_file(cls, config_path: str) -> BdkConfig:
        """Load config from an absolute filepath

        :param config_path: Configuration file absolute path
        :return: Symphony bot configuration object
        """
        config_path = Path(config_path)
        if config_path.exists():
            config_content = config_path.read_text()
            return cls.load_from_content(config_content)
        raise BdkConfigError(f"Config file has not been found at: {config_path.absolute()}")

    @classmethod
    def load_from_content(cls, content: str) -> BdkConfig:
        """Load config from a string containing all config

        :param content: Content of the config file as one string
        :return: Symphony bot configuration object
        """
        data_dict = BdkConfigParser.parse(content)
        return BdkConfig(**data_dict)

    @classmethod
    def load_from_symphony_dir(cls, relative_path: str) -> BdkConfig:
        """Load BdkConfig from a relative path located in the .symphony directory.

        Note: The .symphony directory is located under your home directory.
        It's a convention adopted in order to avoid storing sensitive information (such as usernames, private keys...)
        within the code base.

        :param relative_path: configuration's relative path from the ``$HOME/.symphony`` directory
        :return: Symphony bot configuration object
        """
        config_path = (Path.home() / ".symphony" / relative_path).resolve()
        return cls.load_from_file(str(config_path))


class BdkConfigParser:
    """ Config Parser class

    Provide methods to Deserialize a configuration content
    as a ``str`` in a JSON or YAML format to a BdkConfig object.
    """

    @classmethod
    def parse(cls, config_content: str):
        """

        :param config_content:
        :return:
        """
        try:
            return json.loads(config_content)
        except json.JSONDecodeError:
            pass

        try:
            return yaml.safe_load(config_content)
        except yaml.YAMLError:
            pass

        raise BdkConfigError("Config file is neither in JSON nor in YAML format.")



from symphony.bdk.core.config.model.bdk_retry_config import BdkRetryConfig
from symphony.bdk.gen.agent_api.system_api import SystemApi
from symphony.bdk.gen.agent_api.signals_api import SignalsApi
from symphony.bdk.gen.agent_model.agent_info import AgentInfo

from symphony.bdk.gen.agent_model.v3_health import V3Health


class HealthService:
    """Service class for checking health of the Agent server."""

    def __init__(self, system_api: SystemApi, signals_api: SignalsApi, retry_config: BdkRetryConfig):
        self._system_api = system_api
        self._signals_api = signals_api
        self._retry_config = retry_config

    async def health_check(self) -> V3Health:
        """ Returns the connectivity status of your Agent server.
        Wraps the `Health Check v3 <https://developers.symphony.com/restapi/reference#health-check-v3>`_ endpoint.
        If your Agent server is started and running properly, the status value will be UP.
        Available on Agent 2.57.0 and above.

        :return: V3Health: the connectivity status of your Agent server.
        """
        return await self._system_api.v3_health()

    async def health_check_extended(self) -> V3Health:
        """Returns the connectivity status of the Agent services as well as users connectivity.
        Wraps the
        `Health Check Extended v3 <https://developers.symphony.com/restapi/reference#health-check-extended-v3>`_
        endpoint.
        Available on Agent 2.57.0 and above.

        :return: V3Health: the connectivity status of the Agent services as well as users connectivity.
        """
        return await self._system_api.v3_extended_health()

    async def get_agent_info(self) -> AgentInfo:
        """Gets information about the Agent.
        Wraps the `Agent Info v1 <https://developers.symphony.com/restapi/reference#agent-info-v1>`_ endpoint.
        Available on Agent 2.53.0 and above.

        :return: AgentInfo: information of  the agent server.
        """
        return await self._signals_api.v1_info_get()



class BdkCertificateConfig:
    """Class containing a Certificate configuration for certificate authentication.
    For the format of the certificate file to pass, see
    `ssl.SSLContext.load_cert_chain <https://docs.python.org/3/library/ssl.html#ssl.SSLContext.load_cert_chain>`_.
    For now we only fill the ``certfile`` param, so the ``path`` field should contain the certificate and
    the decrypted private key in PEM format.
    """

    def __init__(self, path: str = None):
        """

        :param path: the path to the client certificate file in PEM format
        """
        self._path = path

    @property
    def path(self) -> str:
        """Path to the client certificate file in PEM format.

        :return: the path to the client certificate file
        """
        return self._path

    @path.setter
    def path(self, certificate_path: str):
        """Sets certificate path.

        :param certificate_path: path to a client certificate in PEM format
        """
        self._path = certificate_path

    def is_valid(self) -> bool:
        """Check if the certificate configuration is valid.

        :return: true if the path to the client certificate is not empty
        """
        return self._path is not None and self._path != ""
